@@ -4,7 +4,10 @@
 import warnings
 
 from numpyro.compat.util import UnsupportedAPIWarning
-from numpyro.primitives import module, param as _param, plate, sample  # noqa: F401
+from numpyro.util import find_stack_level
+
+from numpyro.primitives import module, plate, sample  # noqa: F401 isort:skip
+from numpyro.primitives import param as _param  # noqa: F401 isort:skip
 
 _PARAM_STORE = {}
 
@@ -14,6 +17,7 @@ def get_param_store():
         "A limited parameter store is provided for compatibility with Pyro. "
         "Value of SVI parameters should be obtained via SVI.get_params() method.",
         category=UnsupportedAPIWarning,
+        stacklevel=find_stack_level(),
     )
     # Return an empty dict for compatibility
     return _PARAM_STORE
