@@ -11,6 +11,7 @@ from numpyro.distributions.conjugate import (
     ZeroInflatedNegativeBinomial2,
 )
 from numpyro.distributions.continuous import (
+    CAR,
     LKJ,
     AsymmetricLaplace,
     AsymmetricLaplaceQuantile,
@@ -83,7 +84,7 @@ from numpyro.distributions.distribution import (
     Unit,
 )
 from numpyro.distributions.kl import kl_divergence
-from numpyro.distributions.mixtures import MixtureSameFamily
+from numpyro.distributions.mixtures import Mixture, MixtureGeneral, MixtureSameFamily
 from numpyro.distributions.transforms import biject_to
 from numpyro.distributions.truncated import (
     LeftTruncatedDistribution,
@@ -146,11 +147,14 @@ __all__ = [
     "Logistic",
     "LogNormal",
     "MaskedDistribution",
+    "Mixture",
     "MixtureSameFamily",
+    "MixtureGeneral",
     "Multinomial",
     "MultinomialLogits",
     "MultinomialProbs",
     "MultivariateNormal",
+    "CAR",
     "MultivariateStudentT",
     "LowRankMultivariateNormal",
     "Normal",
