@@ -5,4 +5,8 @@ from numpyro.nn.auto_reg_nn import AutoregressiveNN
 from numpyro.nn.block_neural_arn import BlockNeuralAutoregressiveNN
 from numpyro.nn.masked_dense import MaskedDense
 
-__all__ = ['MaskedDense', 'AutoregressiveNN', 'BlockNeuralAutoregressiveNN']
+__all__ = [
+    "MaskedDense",
+    "AutoregressiveNN",
+    "BlockNeuralAutoregressiveNN",
+]
