@@ -14,10 +14,11 @@ import scipy
 import scipy.stats as osp
 
 import jax
-from jax import grad, lax, tree_map, vmap
+from jax import grad, lax, vmap
 import jax.numpy as jnp
 import jax.random as random
 from jax.scipy.special import expit, logsumexp
+from jax.tree_util import tree_map
 
 import numpyro.distributions as dist
 from numpyro.distributions import (
@@ -164,6 +165,39 @@ _Gaussian2DMixture.reparametrized_params = []
 _Gaussian2DMixture.infer_shapes = lambda *args: (lax.broadcast_shapes(*args), ())
 
 
+def _GeneralMixture(mixing_probs, locs, scales):
+    component_dists = [
+        dist.Normal(loc=loc_, scale=scale_) for loc_, scale_ in zip(locs, scales)
+    ]
+    mixing_distribution = dist.Categorical(probs=mixing_probs)
+    return dist.MixtureGeneral(
+        mixing_distribution=mixing_distribution,
+        component_distributions=component_dists,
+    )
+
+
+_GeneralMixture.arg_constraints = {}
+_GeneralMixture.reparametrized_params = []
+_GeneralMixture.infer_shapes = lambda *args: (lax.broadcast_shapes(*args), ())
+
+
+def _General2DMixture(mixing_probs, locs, cov_matrices):
+    component_dists = [
+        dist.MultivariateNormal(loc=loc_, covariance_matrix=cov_)
+        for loc_, cov_ in zip(locs, cov_matrices)
+    ]
+    mixing_distribution = dist.Categorical(probs=mixing_probs)
+    return dist.MixtureGeneral(
+        mixing_distribution=mixing_distribution,
+        component_distributions=component_dists,
+    )
+
+
+_General2DMixture.arg_constraints = {}
+_General2DMixture.reparametrized_params = []
+_General2DMixture.infer_shapes = lambda *args: (lax.broadcast_shapes(*args), ())
+
+
 class _ImproperWrapper(dist.ImproperUniform):
     def sample(self, key, sample_shape=()):
         transform = biject_to(self.support)
@@ -200,6 +234,29 @@ class FoldedNormal(dist.FoldedDistribution):
         return dist.FoldedDistribution.tree_unflatten(aux_data, params)
 
 
+class _SparseCAR(dist.CAR):
+    reparametrized_params = ["loc", "correlation", "conditional_precision"]
+
+    def __init__(
+        self,
+        loc,
+        correlation,
+        conditional_precision,
+        adj_matrix,
+        *,
+        is_sparse=True,
+        validate_args=None,
+    ):
+        super().__init__(
+            loc,
+            correlation,
+            conditional_precision,
+            adj_matrix,
+            is_sparse=True,
+            validate_args=validate_args,
+        )
+
+
 _DIST_MAP = {
     dist.AsymmetricLaplace: lambda loc, scale, asymmetry: osp.laplace_asymmetric(
         asymmetry, loc=loc, scale=scale
@@ -366,6 +423,48 @@ CONTINUOUS = [
         np.broadcast_to(np.identity(3), (2, 3, 3)),
         None,
     ),
+    T(
+        dist.CAR,
+        1.2,
+        np.array([-0.2, 0.3]),
+        0.1,
+        np.array(
+            [
+                [0.0, 1.0, 1.0, 0.0],
+                [1.0, 0.0, 0.0, 1.0],
+                [1.0, 0.0, 0.0, 1.0],
+                [0.0, 1.0, 1.0, 0.0],
+            ]
+        ),
+    ),
+    T(
+        dist.CAR,
+        np.array([0.0, 1.0, 3.0, 4.0]),
+        0.1,
+        np.array([0.3, 0.7]),
+        np.array(
+            [
+                [0.0, 1.0, 1.0, 0.0],
+                [1.0, 0.0, 0.0, 1.0],
+                [1.0, 0.0, 0.0, 1.0],
+                [0.0, 1.0, 1.0, 0.0],
+            ]
+        ),
+    ),
+    T(
+        _SparseCAR,
+        np.array([[0.0, 1.0, 3.0, 4.0], [2.0, -1.0, -3.0, 2.0]]),
+        0.0,
+        0.1,
+        np.array(
+            [
+                [0.0, 1.0, 1.0, 0.0],
+                [1.0, 0.0, 0.0, 1.0],
+                [1.0, 0.0, 0.0, 1.0],
+                [0.0, 1.0, 1.0, 0.0],
+            ]
+        ),
+    ),
     T(
         dist.MultivariateStudentT,
         15.0,
@@ -510,6 +609,33 @@ CONTINUOUS = [
             ]
         ),  # Covariance
     ),
+    T(
+        _GeneralMixture,
+        np.array([0.2, 0.3, 0.5]),
+        np.array([0.0, 7.7, 2.1]),
+        np.array([4.2, 1.7, 2.1]),
+    ),
+    T(
+        _General2DMixture,
+        np.array([0.2, 0.5, 0.3]),
+        np.array([[-1.2, 1.5], [2.0, 2.0], [-1, 4.0]]),  # Mean
+        np.array(
+            [
+                [
+                    [0.1, -0.2],
+                    [-0.2, 1.0],
+                ],
+                [
+                    [0.75, 0.0],
+                    [0.0, 0.75],
+                ],
+                [
+                    [1.0, 0.5],
+                    [0.5, 0.27],
+                ],
+            ]
+        ),  # Covariance
+    ),
 ]
 
 DIRECTIONAL = [
@@ -977,6 +1103,7 @@ def test_jit_log_likelihood(jax_dist, sp_dist, params):
         "_ImproperWrapper",
         "LKJ",
         "LKJCholesky",
+        "_SparseCAR",
     ):
         pytest.xfail(reason="non-jittable params")
 
@@ -1331,6 +1458,9 @@ def test_log_prob_gradient(jax_dist, sp_dist, params):
 
     eps = 1e-3
     for i in range(len(params)):
+        if jax_dist is _SparseCAR and i == 3:
+            # skip taking grad w.r.t. adj_matrix
+            continue
         if isinstance(
             params[i], dist.Distribution
         ):  # skip taking grad w.r.t. base_dist
@@ -1461,6 +1591,8 @@ def test_mean_var(jax_dist, sp_dist, params):
     else:
         if jnp.all(jnp.isfinite(d_jax.mean)):
             assert_allclose(jnp.mean(samples, 0), d_jax.mean, rtol=0.05, atol=1e-2)
+        if isinstance(d_jax, dist.CAR):
+            pytest.skip("CAR distribution does not have `variance` implemented.")
         if jnp.all(jnp.isfinite(d_jax.variance)):
             assert_allclose(
                 jnp.std(samples, 0), jnp.sqrt(d_jax.variance), rtol=0.05, atol=1e-2
@@ -1477,6 +1609,8 @@ def test_distribution_constraints(jax_dist, sp_dist, params, prepend_shape):
         _TruncatedCauchy,
         _GaussianMixture,
         _Gaussian2DMixture,
+        _GeneralMixture,
+        _General2DMixture,
     ):
         pytest.skip(f"{jax_dist.__name__} is a function, not a class")
     dist_args = [p for p in inspect.getfullargspec(jax_dist.__init__)[0][1:]]
@@ -2225,6 +2359,15 @@ def test_sine_bivariate_von_mises_batch_shape(batch_shape):
     assert samples.shape == (*batch_shape, 2)
 
 
+def test_sine_bivariate_von_mises_sample_mean():
+    loc = jnp.array([[2.0, -1.0], [-2, 1.0]])
+
+    sine = SineBivariateVonMises(*loc, 5000, 5000, 0.0)
+    samples = sine.sample(random.PRNGKey(0), (5000,))
+
+    assert_allclose(_circ_mean(samples).T, loc, rtol=5e-3)
+
+
 @pytest.mark.parametrize("batch_shape", [(), (4,)])
 def test_polya_gamma(batch_shape, num_points=20000):
     d = dist.TruncatedPolyaGamma(batch_shape=batch_shape)
@@ -2417,3 +2560,13 @@ def test_kl_dirichlet_dirichlet(shape):
     x = p.sample(random.PRNGKey(0), (10_000,)).copy()
     expected = jnp.mean((p.log_prob(x) - q.log_prob(x)), 0)
     assert_allclose(actual, expected, rtol=0.05)
+
+
+def test_vmapped_binomial_p0():
+    # test that vmapped binomial with p = 0 does not have an infinite loop
+    def sample_binomial_withp0(key):
+        n = 2 * (random.uniform(key) > 0.5)
+        _, key = random.split(key)
+        return dist.Binomial(total_count=n, probs=0).sample(key)
+
+    jax.vmap(sample_binomial_withp0)(random.split(random.PRNGKey(0), 1))
