@@ -6,8 +6,14 @@ from numpy.testing import assert_allclose, assert_raises
 import pytest
 
 import jax
-from jax import jit, random, tree_map, value_and_grad, vmap
+from jax import jit, random, value_and_grad, vmap
 import jax.numpy as jnp
+from jax.tree_util import tree_map
+
+try:
+    import funsor
+except ImportError:
+    funsor = None
 
 import numpyro
 from numpyro import handlers
@@ -657,6 +663,7 @@ def test_lift_memoize():
             model()
 
 
+@pytest.mark.skipif(funsor is None, reason="require funsor installation")
 def test_collapse_beta_binomial():
     total_count = 10
     data = 3.0
@@ -695,6 +702,7 @@ def test_collapse_beta_binomial():
     assert_allclose(params1["c0"], params2["c0"])
 
 
+@pytest.mark.skipif(funsor is None, reason="require funsor installation")
 def test_collapse_beta_bernoulli():
     data = 0.0
 
